<?php
// $Id: build_file_paths.inc,v 1.1.2.2 2010/05/21 15:53:41 dvessel Exp $

/**
 * Scans for files based on regex patterns and returns an array of paths.
 * 
 * The scanning starts from base themes to connected sub-themes which are
 * currently active. This will ignore plug-ins.
 * 
 * @param $patterns
 *  Regex pattern to be compared with the name of the file. String or an array
 *  of strings are accepted.
 * @param $ignore_patterns
 *  Regex patterns to be compared with the full path including the name of the
 *  file. Any matches will be ignored. String or an array of strings are
 *  accepted.
 * @param $theme_offset
 *  This signals at which base or subtheme to start scanning as it digs further
 *  into connected subthemes.
 */
function hex_theme_file_scan($patterns, $ignore_patterns = NULL, $theme_offset = NULL) {

  foreach (list_themes() as $theme => $theme_data) {
    $theme_paths[$theme] = dirname($theme_data->filename);
  }
  foreach (hex_plugin_list('all') as $plugin) {
    $plugin_paths[] = hex_plugin_path($plugin);
  }

  // This path will be cut down in the next loop.
  $_ignore_paths = $theme_paths;

  $found_files = array();

  $start_from_theme_key = $theme_offset;
  // Sorted from base themes to their child sub-themes.
  // This scanning order is necessary to guarantee the correct order in collecting paths.
  foreach (hex_active_themes() as $theme_key) {
    // This will be used to prevent the base theme from scanning child themes or any
    // other theme that could possibly be inactive and live within the base theme.
    unset($_ignore_paths[$theme_key]);

    // If there's an offset and it does not match, skip the loop until it matches.
    if (isset($start_from_theme_key) && $theme_key != $start_from_theme_key) {
      continue;
    }
    elseif (isset($start_from_theme_key) && $theme_key == $start_from_theme_key) {
      unset($start_from_theme_key);
    }

    // Scan theme files but ignore plugins, inactive themes and sub-themes.
    // Subthemes will be scanned in subsequent loops.
    foreach (hex_file_scan(hex_theme_path($theme_key), $patterns, $_ignore_paths + $plugin_paths, $ignore_patterns) as $paths) {
      $found_files = array_merge($found_files, $paths);
    }
  }

  return $found_files;
}

/**
 * Build paths to files based on regex patterns set in the .info or .pinfo file.
 * 
 * The returned paths are keyed according to the search scope. Possible scopes
 * are "theme", "plugin" or "all".
 * 
 * Patterns set from base themes are inherited into their children. the same
 * applies to plug-in patterns. If the host theme is a parent to another theme
 * the plug-in patterns can influence any child themes. This also depends on
 * the scope of the search.
 */
function _hex_build_file_paths() {

  foreach (list_themes() as $theme => $theme_data) {
    $theme_paths[$theme] = dirname($theme_data->filename);
  }
  foreach (hex_plugin_list('all') as $plugin) {
    $plugin_paths[$plugin] = hex_plugin_path($plugin);
  }
  // Build list of disabled plug-in paths to be ignored.
  // Also build a list of active plug-in paths.
  $plugin_paths_inactive = $plugin_paths;
  foreach (hex_plugin_list() as $plugin) {
    unset($plugin_paths_inactive[$plugin]);
  }
  $ignore_all_plugins = array_values($plugin_paths);
  $ignore_inactive_plugins = array_values($plugin_paths_inactive);

  // This path will be cut down with each itteration in the following loop
  // to prevent unintentional scanning of unconnected themes and to force
  // a specific order while discovering files.
  $_ignore_paths = $theme_paths;

  $discovered_files = $patterns = array();

  foreach (hex_active_themes() as $theme) {

    unset($_ignore_paths[$theme]);
    $ignore_subthemes = array_values($_ignore_paths);

    // Collect patterns!
    foreach (array('file patterns', 'ignore patterns') as $pattern_type) {

      // Look for theme defined patterns first.
      foreach (hex_theme_info($pattern_type, array(), $theme) as $set => $pattern) {
        // Define search scope as pattern set.
        $search_scopes = array('theme');
        if (!is_string($set)) {
          $set = 'general';
        }
        elseif (strpos($set, ':') !== FALSE) {
          $search_scopes = explode(':', $set);
          $set = array_pop($search_scopes);
        }
        foreach ($search_scopes as $scope) {
          $patterns[$scope][$set][$pattern_type][] = $pattern;
        }
      }

      // Do the same for plug-in's to allow their patterns to influence themes.
      // This will not find files hosted within plug-in's though. That will be
      // processed separately.
      $current_theme_plugin_paths = array();
      foreach (hex_plugin_list_by_owner($theme) as $plugin) {
        $current_theme_plugin_paths[$plugin] = $plugin_paths[$plugin];
        foreach (hex_plugin_info($plugin, $pattern_type, array(), $theme) as $set => $pattern) {

          // Define search scope as pattern set.
          $search_scopes = array('plugin');
          if (!is_string($set)) {
            $set = 'general';
          }
          elseif (strpos($set, ':') !== FALSE) {
            $search_scopes = explode(':', $set);
            $set = array_pop($search_scopes);
          }
          foreach ($search_scopes as $scope) {
            $patterns[$scope][$set][$pattern_type][] = $pattern;
          }

        }
      }
    }

    // Scan with collected patterns!
    foreach ($patterns as $scope => $pattern_sets) {
      foreach ($pattern_sets as $set => $pattern) {
        if (!isset($found_files[$scope][$set])) {
          $found_files[$scope][$set] = array();
        }

        if (!isset($pattern['file patterns'])) {
          continue;
        }
        if (!isset($pattern['ignore patterns'])) {
          $pattern['ignore patterns'] = array();
        }

        if ($scope == 'all' || $scope == 'theme') {
          foreach (hex_file_scan($theme_paths[$theme], $pattern['file patterns'], array_merge($ignore_subthemes, $ignore_all_plugins), $pattern['ignore patterns']) as $file_name => $paths) {
            if (isset($found_files[$scope][$set][$file_name])) {
              $found_files[$scope][$set][$file_name] = array_merge($found_files[$scope][$set][$file_name], $paths);
            }
            else {
              $found_files[$scope][$set][$file_name] = $paths;
            }
            // Add to theme scope key so it can be selective when called with
            // the scope parameter in hex_path_to_file() and related functions.
            if ($scope == 'all') {
              if (isset($found_files['theme'][$set][$file_name])) {
                $found_files['theme'][$set][$file_name] = array_merge($found_files[$scope][$set][$file_name], $paths);
              }
              else {
                $found_files['theme'][$set][$file_name] = $paths;
              }
            }
          }
        }

        if ($scope == 'all' || $scope == 'plugin') {
          foreach ($current_theme_plugin_paths as $plugin => $path) {
            foreach (hex_file_scan($path, $pattern['file patterns'], $ignore_inactive_plugins, $pattern['ignore patterns']) as $file_name => $paths) {
              if (isset($found_files[$scope][$set][$file_name])) {
                $found_files[$scope][$set][$file_name] = array_merge($found_files[$scope][$set][$file_name], $paths);
              }
              else {
                $found_files[$scope][$set][$file_name] = $paths;
              }
              $found_files[$scope][$set][$file_name] = array_unique($found_files[$scope][$set][$file_name]);

              // Add to theme scope key so it can be selective when called with
              // the scope parameter in hex_path_to_file() and related functions.
              if ($scope == 'all') {
                if (isset($found_files['plugin'][$set][$file_name])) {
                  $found_files['plugin'][$set][$file_name] = array_merge($found_files[$scope][$set][$file_name], $paths);
                }
                else {
                  $found_files['plugin'][$set][$file_name] = $paths;
                }
                $found_files['plugin'][$set][$file_name] = array_unique($found_files[$scope][$set][$file_name]);
              }
              
            }
          }
        }

      }
    }
  }

  return $found_files;
}

/**
 * Pass a path and regex pattern, get an array of file paths in return.
 * 
 * @param $path
 * @param $patterns
 * @param $ignore_paths
 * @param $ignore_patterns
 */
function hex_file_scan($path, $patterns, $ignore_paths = NULL, $ignore_patterns = NULL) {

  $found_files = array();

  if (is_array($patterns)) {
    $patterns = implode('|', $patterns);
  }
  if (is_array($ignore_patterns)) {
    $ignore_patterns = implode('|', $ignore_patterns);
  }
  if (!isset($ignore_paths)) {
    $ignore_paths = array();
  }

  if ($scan_data = file_scan_directory($path, $patterns)) {
    foreach ($scan_data as $full_path => $file) {
      // Any directory or file prefixed with a tilde will be ignored. This
      // matches the behavior of Photoshop plug-ins and seems like a good way
      // of quickly disabling files or groups of files. Files and directories
      // with leading dots also ignored since they are hidden.
      if (strpos($file->filename, '/~') !== FALSE || strpos($file->filename, '/.') !== FALSE) {
        continue;
      }
      foreach ($ignore_paths as $ignore_path) {
        if (strpos($file->filename, $ignore_path) === 0) {
          // Skip to next file.
          continue 2;
        }
      }
      // ereg() used to match Drupal's use in file_scan_directory().
      // Change to preg_match() in Drupal 7.
      if (!(!empty($ignore_patterns) && ereg($ignore_patterns, $file->basename))) {
        $found_files[$file->basename][] = $full_path;
      }
    }
  }

  return $found_files;
}


