<?php
// $Id: build_theme_registry.inc,v 1.1.2.2 2010/05/21 15:53:41 dvessel Exp $

/**
 * Implementation of HOOK_theme().
 */
function hex_theme() {
  return array(
    // Core bug missing the last two arguments. Arguments must be properly
    // registered when any theme based on Hexagon overrides the hook.
    'menu_item' => array(
      'arguments' => array('link' => NULL, 'has_children' => NULL, 'menu' => '', 'in_active_trail' => FALSE, 'extra_class' => NULL),
    ),
  );
}

/**
 * Does the following modifications:
 * 
 * - Adds supporting variable preprocess include files and files for theme
 *   functions so they are more easily separated and organized. When it is
 *   included depends on Drupal core version.
 * 
 * - Adds preprocess functions for theme functions. See the code comments in
 *   the examples at 'theme-hooks/theme-examples' within the starter theme.
 * 
 * - Stores paths to template suggestions. This bypasses core's template
 *   suggestions discovery. Pairing a base template with its alternate is no
 *   longer necessary due to this change.
 */
function _hex_theme_registry_alter(&$registry) {

  // If the modified registry doesn't behave as expected. Uncomment this to
  // inspect all modules which alter the theme registry. The modifications
  // by Hexagon shouldn't be a problem but there could possibly be conflicts.
  // Functions to look into will be named "MODULE_NAME_theme_registry_alter".
  // print var_dump(module_implements('theme_registry_alter'));

  // Collect paths and use them to match the owners for each theming hook.
  $path_to_owner = array();
  foreach (array('module' => module_implements('theme'), 'theme' => hex_active_themes()) as $type => $owners) {
    foreach ($owners as $owner_key) {
      $path_to_owner[drupal_get_path($type, $owner_key)] = $owner_key;
    }
  }
  foreach (hex_plugin_list() as $plugin) {
    $path_to_owner[hex_plugin_path($plugin)] = $plugin;
  }

  // Prepare files for auto includes.
  $auto_includes = array();
  foreach (hex_group_file_paths('theme_hook_includes', 'all') as $file_path) {
    $auto_includes[basename($file_path)][] = $file_path;
  }

  // This will be used to find template suggestions.
  $theme_templates = array();
  foreach (hex_group_file_paths('theme_hook_templates') as $file_path) {
    $theme_templates[basename($file_path)] = $file_path;
  }

  // Gather prefixes in the correct order.
  $prefixes = array('template' => 'template');
  $prefixes += module_list();
  $prefixes += hex_active_themes();
  $prefixes += hex_plugin_list();

  // START REGISTRY EDIT /////////////////////////////////////////////////////

  foreach ($registry as $hook => $info) {

    // State of implementation type.
    $hook_is_function = isset($info['function']) && !isset($info['template']);
    $hook_is_template = !$hook_is_function;
    // Implementation level.
    $type_is_module = $info['type'] == 'module';
    $type_is_theme  = !$type_is_module;

    // Get the current owner.
    $current_owner  = $path_to_owner[$info['theme path']];
    // And the original implementor of this hook.
    if (isset($path_to_owner[$info['theme paths'][0]])) {
      $original_owner = $path_to_owner[$info['theme paths'][0]];
    }
    else {
      // Cut down the path until the owner is found.
      $path = $info['theme paths'][0];
      while ($pos = strrpos($path, '/')) {
        $path = substr($path, 0, $pos);
        if (isset($path_to_owner[$path])) {
          $original_owner = $path_to_owner[$path];
          break;
        }
      }
    }

    // PROCESS INCLUDES! /////////////////////////////////////////////////////

    // Set-up all possible prefixes that are associated with this hook.
    $name_prefixes = array($hook);
    // Make sure theming patterns include the originating hook.
    if (isset($info['original hook'])) {
      $name_prefixes[] = $info['original hook'];
    }

    // Add include files based on matching names.
    $include_files = array();
    foreach ($name_prefixes as $prefix) {
      $name_prefix = str_replace('_', '-', $prefix);
      if (isset($info['function']) && isset($auto_includes[$name_prefix . '.func.inc'])) {
        $include_files = array_merge($include_files, $auto_includes[$name_prefix . '.func.inc']);
      }
      if (isset($auto_includes[$name_prefix . '.vars.inc'])) {
        $include_files = array_merge($include_files, $auto_includes[$name_prefix . '.vars.inc']);
      }
    }
    $original_owner_name = str_replace('_', '-', $original_owner);
    if (isset($auto_includes[$original_owner_name . '.theme.inc'])) {
      $include_files = array_merge($include_files, $auto_includes[$original_owner_name . '.theme.inc']);
    }

    // Versions of Drupal Core before 6.16 will not have this set.
    if (!isset($registry[$hook]['include files'])) {
      $registry[$hook]['include files'] = $include_files;
    }
    else {
      $registry[$hook]['include files'] = array_merge($registry[$hook]['include files'], $include_files);
    }

    // This will be used to see if the registry has any changes for modules at the end of this loop.
    $state_for_module_theme_function_hook = $hook_is_function && $type_is_module ? serialize($registry[$hook]) : FALSE;

    // Flag for compatibility with includes files.
    // Version of core before 6.16 cannot dynamically include files in theme() calls.
    $includes_compatibility = !empty($include_files) && (float) VERSION <= 6.15;

    /**
     * Allow variable process functions for theme functions and add in template
     * suggestion paths for theming hooks implemented as templates.
     */
    if ($hook_is_function) {
      // HOOK IS FUNCTION! ///////////////////////////////////////////////////

      $registry[$hook]['preprocess functions'] = array();
      if ($includes_compatibility) {
        $registry[$hook]['preprocess functions'][] = 'hex_process_includes';
      }

      foreach ($prefixes as $prefix) {
        if (function_exists($prefix .'_func_preprocess_'. $hook)) {
          $registry[$hook]['preprocess functions'][] = $prefix .'_func_preprocess_'. $hook;
        }
        if (!empty($info['original hook']) && function_exists($prefix .'_func_preprocess_'. $info['original hook'])) {
          $registry[$hook]['preprocess functions'][] = $prefix .'_func_preprocess_'. $info['original hook'];
        }
      }

      if (!empty($registry[$hook]['preprocess functions'])) {
        $registry[$hook]['preprocess functions'][] = 'hex_func_preprocess_finalize';
        $registry[$hook]['arguments']['-hex']->hook = $hook;
        $registry[$hook]['arguments']['-hex']->type = 'function';
        $registry[$hook]['arguments']['-hex']->owner_type = $type_is_module ? 'module' : 'theme';
        $registry[$hook]['arguments']['-hex']->original_owner = $original_owner;
        $registry[$hook]['arguments']['-hex']->current_owner  = $current_owner;
        $registry[$hook]['arguments']['-hex']->function  = $info['function'];
        $registry[$hook]['arguments']['-hex']->arguments = !empty($info['arguments']) ? array_keys($info['arguments']) : array();
        $registry[$hook]['arguments']['-hex']->post_render_callback = '';
        unset($registry[$hook]['function']);
        // To prevent notices.
        $registry[$hook]['template'] = 'null';
      }

    }
    else {
      // HOOK IS TEMPLATE! ///////////////////////////////////////////////////

      /**
       * ADD VARIABLE PROCESSORS.
       * 
       * Ordered into three phases.
       */
      $variable_processors = array(
        'pre_preprocess' => array(),
        'preprocess'     => $info['preprocess functions'],
        're_process'     => array(),
      );
      foreach (array('pre_preprocess', 'preprocess', 're_process') as $phase) {
        foreach ($prefixes as $prefix) {
          if (function_exists($prefix . '_' . $phase)) {
            $variable_processors[$phase][] = $prefix . '_' . $phase;
          }
          if (function_exists($prefix . '_' . $phase . '_' . $hook)) {
            $variable_processors[$phase][] = $prefix . '_' . $phase . '_' . $hook;
          }
        }
      }
      $preprocess_functions = array();
      foreach ($variable_processors as $phase => $preprocessors) {
        $preprocess_functions = array_merge($preprocess_functions, $preprocessors);
      }
      // Force uniqueness and set back to registry.
      $registry[$hook]['preprocess functions'] = array_unique($preprocess_functions);

      if ($includes_compatibility) {
        array_unshift($registry[$hook]['preprocess functions'], 'hex_process_includes');
      }
      $registry[$hook]['preprocess functions'][] = 'hex_tpl_preprocess_finalize';

      // First create a filter so templates beginning with a similar prefix
      // as the current hook is not mistakenly added.
      $tpl_filter = array();
      foreach (array_keys($registry) as $key) {
        if (strpos($key, $hook) === 0 && $key != $hook) {
          $tpl_filter[] = str_replace('_', '-', $key);
        }
      }

      // Filter for template suggestions.
      $template_alts = array();
      foreach ($theme_templates as $name => $template_path) {
        if (preg_match('/^' . str_replace('_', '-', $hook) . '[a-zA-Z0-9_-]+\.tpl\.php$/', $name)) {
          if (!empty($tpl_filter) && preg_match('/^(' . implode('|', $tpl_filter) . ')[a-zA-Z0-9_-]*\.tpl\.php$/', $name)) {
            continue;
          }
          $template_alts[basename($name, '.tpl.php')] = $template_path;
        }
      }

      $registry[$hook]['arguments']['-hex']->hook = $hook;
      $registry[$hook]['arguments']['-hex']->type = 'template';
      $registry[$hook]['arguments']['-hex']->owner_type = $type_is_module ? 'module' : 'theme';
      $registry[$hook]['arguments']['-hex']->original_owner = $original_owner;
      $registry[$hook]['arguments']['-hex']->current_owner  = $current_owner;
      $registry[$hook]['arguments']['-hex']->arguments = !empty($info['arguments']) ? array_keys($info['arguments']) : array();
      $registry[$hook]['arguments']['-hex']->template_alts = $template_alts;
      $registry[$hook]['arguments']['-hex']->post_render_callback = '';
    }

    // This "type" is changed to allow an alternate render function invoked
    // from core's theme() function. When type is "module" and the hook type
    // is a *function*, the theme adding variable preprocessors for the function
    // not hosted in the theme needs this alternate renderer. The alternate
    // render is the phptemplate_render_template() function.
    // 
    // This very selective behavior is to prevent any possible performance
    // degradation for theme functions not hosted in or influenced by the
    // running themes.
    // 
    // Using serialize -a function not meant for this purpose- to compare the
    // multidimentional array but it seems any performance degradation would be
    // noticed only when unserializing very large data sets. Something we aren't
    // doing here. And this will seldom run anyways.
    if ($state_for_module_theme_function_hook && $state_for_module_theme_function_hook != serialize($registry[$hook])) {
      $registry[$hook]['type'] = 'base_theme';
    }
    // Hook templates owned by modules are always changed.
    elseif ($hook_is_template && $type_is_module) {
      $registry[$hook]['type'] = 'base_theme';
    }
  }

}

/**
 * Rebuild and save the modified theme registry.
 * 
 * This allows the theme to get the whole scope of the registry unlike
 * HOOK_theme(). It shouldn't break anything and the rest of the theming system
 * will work as it's designed.
 * 
 * This will be called before cores' _theme_load_registry() and allow it to
 * function returning a modified version of the registry. All files should be
 * included to allow discovery of theme functions and variable processors.
 * 
 * The core function _theme_build_registry() will not be invoked. This takes
 * its place.
 */
function _hex_build_registry() {
  global $theme_info, $base_theme_info, $theme_engine;
  $theme = $theme_info;
  $base_theme = $base_theme_info;

  ////////////////////////////////////////////////////////////////////////////
  // COPY START _theme_build_registry() //////////////////////////////////////

  $cache = array();
  // First, process the theme hooks advertised by modules. This will
  // serve as the basic registry.
  foreach (module_implements('theme') as $module) {
    _theme_process_registry($cache, $module, 'module', $module, drupal_get_path('module', $module));
  }

  // Process each base theme.
  foreach ($base_theme as $base) {
    // If the base theme uses a theme engine, process its hooks.
    $base_path = dirname($base->filename);
    if ($theme_engine) {
      _theme_process_registry($cache, $theme_engine, 'base_theme_engine', $base->name, $base_path);
    }
    _theme_process_registry($cache, $base->name, 'base_theme', $base->name, $base_path);
  }

  // And then the same thing, but for the theme.
  if ($theme_engine) {
    _theme_process_registry($cache, $theme_engine, 'theme_engine', $theme->name, dirname($theme->filename));
  }

  // Finally, hooks provided by the theme itself.
  _theme_process_registry($cache, $theme->name, 'theme', $theme->name, dirname($theme->filename));

  // COPY END _theme_build_registry() ////////////////////////////////////////
  ////////////////////////////////////////////////////////////////////////////

  // Let Hexagon plug-ins define hooks. The reason why _theme_build_registry()
  // had to be copied. To fit this in before drupal_alter().
  foreach (hex_plugin_list() as $plugin) {
    _theme_process_registry($cache, $plugin, 'plugin', $plugin, hex_plugin_path($plugin));
  }

  // Let modules alter the registry
  drupal_alter('theme_registry', $cache);

  // Final alterations, save then set.
  _hex_theme_registry_alter($cache);
  _theme_save_registry($theme_info, $cache);
  _theme_set_registry($cache);

}
