<?php
// $Id: theme_hook_process.inc,v 1.1.2.2 2010/05/21 06:42:02 dvessel Exp $


/**
 * PRE-PRE-PROCESS -> PRE-PROCESS -> RE-PROCESS (variable processors)
 * ---------------------------------------------------------------------------
 * This theme allows three distinct phases of variable processors to work.
 * Drupal core normally allows only one phase named "preprocess". Hexagon allows
 * the following:
 *  - pre_preprocess      - Early phase.
 *  - preprocess          - Normally allowed in core. Will be invoked as usual.
 *  - re_process          - Late phase.
 * 
 * Then there is the implementation level indicated by the prefix.
 *  - template            - Normally the owner of the hook implements this.
 *  - MODULE_NAME         - Module tapping into the hook. Not the owner.
 *  - ENGINE_NAME_engine  - Implemented by the theme engine itself. Not used.
 *  - ENGINE_NAME         - Engine name used by the theme. Not recommended.
 *  - THEME_NAME          - Theme implementation.
 *  - SUBTHEME_NAME       - Sub-theme implementation.
 *  - PLUGIN_NAME         - Plugin implementation.
 * 
 * "page" hook example in order of execution:
 *  - hex_pre_preprocess
 *  - template_preprocess
 *  - template_preprocess_page
 *  - hex_re_process
 * 
 * @see http://drupal.org/node/223430
 *  How preprocess functions normally work.
 * 
 * @see http://drupal.org/node/306358
 *  Drupal 7 includes the second phase variable processor but they are named
 *  "process" not "re_process" as it's done here in the late phase. Process is
 *  too commonly used for other functions and conflicts may occur. It will be
 *  safe to change when the theme is upgraded for 7.
 * 
 */


/**
 * These variables will be available to all templates, everywhere!
 */
function hex_pre_preprocess(&$variables, $hook) {
  // Initialize id and class attributes.
  if (!isset($variables['id_attribute'])) {
    $variables['id_attribute'] = '';
  }
  $variables['classes_array'][] = $hook;
  // Reference variables so they can be added either way.
  $variables['html_elements']['#attributes']['id']    = &$variables['id_attribute'];
  $variables['html_elements']['#attributes']['class'] = &$variables['classes_array'];
}

/**
 * All this does is include supporting files when running Drupal core versions
 * before 6.16. 6.16 contains a fix that automatically includes files from
 * theme() function calls. This is only here to maintain compatibility.
 */
function hex_process_includes(&$variables, $hook) {
  static $processed;
  if (!isset($processed[$hook])) {
    $processed[$hook] = TRUE;
    $registry = theme_get_registry();
    // Include dependent files.
    if (!empty($registry[$hook]['include files'])) {
      foreach ($registry[$hook]['include files'] as $include_path) {
        include_once $include_path;
      }
    }
  }
}

/**
 * Last preprocess function for templates.
 */
function hex_tpl_preprocess_finalize(&$variables, $hook) {

  if (isset($variables['template_files'])) {
    $suggestions = $variables['template_files'];
  }
  if (isset($variables['template_file'])) {
    $suggestions[] = $variables['template_file'];
  }

  $variables['-hex']->template_file = '';
  // This allows templates to not be paired with the base template.
  if (!empty($suggestions) && !empty($variables['-hex']->template_alts)) {
    // Loop through suggestions in FIFO order.
    foreach (array_reverse($suggestions) as $suggestion) {
      if (isset($variables['-hex']->template_alts[$name . '.tpl.php'])) {
        $variables['-hex']->template_file = $variables['-hex']->template_alts[$name . '.tpl.php'];
        // These are removed so Core doesn't have to go searching for what's been found over here.
        unset($variables['template_files']);
        unset($variables['template_file']);
        break;
      }
    }
  }

}

/**
 * Last preprocess function used only for theme functions.
 * 
 * This is where the original theme function is called.
 */
function hex_func_preprocess_finalize(&$variables, $hook) {

  if (isset($variables['template_file'])) {
    unset($variables['template_file']);
  }
  if (isset($variables['template_files'])) {
    unset($variables['template_files']);
  }

  // Set expected arguments for the theme function.
  $args = array();
  foreach ($variables['-hex']->arguments as $arg_key) {
    $args[] = $variables[$arg_key];
  }
  $variables['-hex']->contents = call_user_func_array($variables['-hex']->function, $args);

}

/**
 * Render a template or pass through content set for theme functions using
 * variable preprocessors.
 * 
 * Optional post render callback can be set. Useful for wrapping any piece of
 * content even if it doesn't live in the theme.
 */
function phptemplate_render_template($template_file, $variables) {

  if (isset($variables['-hex']->function)) {
    $contents = $variables['-hex']->contents;
  }
  else {
    if (!empty($variables['-hex']->template_file)) {
      $template_file = $variables['-hex']->template_file;
    }
    extract($variables, EXTR_REFS);  // Extract the variables to a local namespace
    ob_start();                      // Start output buffering
    include "./$template_file";      // Include the template file
    $contents = ob_get_contents();   // Get the contents of the buffer
    ob_end_clean();                  // End buffering and discard
  }

  if (!empty($variables['-hex']->post_render_callback)) {
    $contents = call_user_func($variables['-hex']->post_render_callback, $variables, $contents);
  }

  return $contents;
}
